;==============================================================================
;*Author:  Jonathan Weaver, jonw0224@netscape.net
;E-mail Contact: jonw0224@netscape.net
;*Description:  Delay routines for delays of 4 cycles and up.
;*Version: 1.1
;*Date: 10/11/2005
;*Filename: delay.inc
;
;Copyright (C) 2005 Jonathan Weaver
;
;This program is free software; you can redistribute it and/or modify it under
;the terms of the GNU General Public License as published by the Free Software
;Foundation; either version 2 of the License, or (at your option) any later
;version.
;
;This program is distributed in the hope that it will be useful, but WITHOUT
;ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;details.
;
;You should have received a copy of the GNU General Public License along with
;this program; if not, write to the Free Software Foundation, Inc.,
;51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
;
;Revision History:
; v1.0 - 2/16/2005 - Created file
; v1.0 - 10/10/2005 - Added procedure delays less than 15 cycles.
; v1.1 - 10/11/2005 - Separated macros (delayH.inc) and routines (delay.inc)
;
;==============================================================================

ifdef _ContDelay        ;Conditional compile
;*Delays for the number of cycles equal to WREG.  WREG must contain a value of 14 or more.  Destroys value in WREG.  Affects STATUS register.  Compiles when _ContDelay has been defined.  Requires 15 program words
Delay	;public
        addlw -D'14'
DelayLoop
        addlw -4
        btfsc STATUS, C
            goto DelayLoop
        addlw 4             ;1
        btfsc STATUS, Z     ;2
            goto DelayL1    ;was -4, 4 => 8 cycles
        addlw -1            ;4
        btfsc STATUS, Z     ;5
            goto DelayL2    ;was -3, 7 => 9 cycles
        addlw -1            ;7
        btfsc STATUS, Z     ;8
            return          ;was -1, 10 => 10 cycles
DelayL2
        return              ;11 cycles, was -1
DelayL1
        goto DelayL2
endif               ;End conditional compile

ifdef _ProcDelay	;Conditional compile
;*Delay constant routine.  Compiles if _ProcDelay is defined.  Requires 1 program word plus Delay13 for routine plus 1 program word for each call statement.
Delay14 ;public
	nop
;*Delay constant routine.  Compiles if _ProcDelay is defined.  Requires 1 program word plus Delay12 for routine plus 1 program word for each call statement.
Delay13 ;public
	nop
;*Delay constant routine.  Compiles if _ProcDelay is defined.  Requires 1 program word plus Delay11 for routine plus 1 program word for each call statement.
Delay12 ;public
	nop
;*Delay constant routine.  Compiles if _ProcDelay is defined.  Requires 1 program word plus Delay10 for routine plus 1 program word for each call statement.
Delay11 ;public
	nop
;*Delay constant routine.  Compiles if _ProcDelay is defined.  Requires 1 program word plus Delay9 for routine plus 1 program word for each call statement.
Delay10 ;public
	nop
;*Delay constant routine.  Compiles if _ProcDelay is defined.  Requires 1 program word plus Delay8 for routine plus 1 program word for each call statement.
Delay9  ;public
	nop
;*Delay constant routine.  Compiles if _ProcDelay is defined.  Requires 1 program word plus Delay7 for routine plus 1 program word for each call statement.
Delay8  ;public
	nop
;*Delay constant routine.  Compiles if _ProcDelay is defined.  Requires 1 program word plus Delay6 for routine plus 1 program word for each call statement.
Delay7  ;public
	nop
;*Delay constant routine.  Compiles if _ProcDelay is defined.  Requires 1 program word plus Delay5 for routine plus 1 program word for each call statement.
Delay6  ;public
	nop
;*Delay constant routine.  Compiles if _ProcDelay is defined.  Requires 1 program word plus Delay4 for routine plus 1 program word for each call statement.
Delay5  ;public
	nop
;*Delay constant routine.  Compiles if _ProcDelay is defined.  Requires 1 program word for routine plus 1 program word for each call statement.
Delay4  ;public
	return
endif				;End conditional compile