;==============================================================================
;*Author:  Jonathan Weaver, jonw0224@netscape.net
;E-mail Contact: jonw0224@netscape.net
;*Description:  Delay macros for delays of 1 cycle and up.  Works with delay.inc.
;*Version: 1.1
;*Date: 10/11/2005
;*Filename: delayH.inc
;
;Copyright (C) 2005 Jonathan Weaver
;
;This program is free software; you can redistribute it and/or modify it under
;the terms of the GNU General Public License as published by the Free Software
;Foundation; either version 2 of the License, or (at your option) any later
;version.
;
;This program is distributed in the hope that it will be useful, but WITHOUT
;ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;details.
;
;You should have received a copy of the GNU General Public License along with
;this program; if not, write to the Free Software Foundation, Inc.,
;51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
;
;Revision History:
; v1.0 - 2/16/2005 - Created file
; v1.0 - 10/10/2005 - Added procedure delays less than 15 cycles.
; v1.1 - 10/11/2005 - Separated macros (delayH.inc) and subroutines (delay.inc)
;
;==============================================================================

;Handles delays of larger than 256.  Not to be called directly, DELAY will call automatically.
LARGEDELAY macro lit
	local dcnt
	dcnt = lit
	while dcnt > D'256'
		DELAY D'256'
		dcnt = dcnt - D'256'
	endw
	if dcnt > 0
		DELAY dcnt
	endif
endm

;Handles delays of larger than 1024.  Not to be called directly, DELAY will call automatically
LARGEDELAYB macro lit
	local dcnt
	dcnt = lit
	while dcnt > D'1024'
		DELAY D'1024'
		dcnt = dcnt - D'1024'
	endw
	if dcnt > 0
		DELAY dcnt
	endif
endm

;------------------------------------------------------------------------------
;*Delays for the number of cycles equal to lit.  DELAY will create optimized delay code and will use the Delay routines if _contDelay or _procDelay are defined.
;*lit: lit can be any number of cycles to delay.  If lit > 11 then generated code may affect WREG and STATUS.
DELAY macro lit ;public
    local dcnt
    dcnt = lit
	if dcnt > D'256'
		ifdef _ContDelay
			LARGEDELAY dcnt
		else
			if dcnt > D'1024'
				LARGEDELAYB dcnt
			else
				movlw dcnt/4 - 1
				addlw -1
				btfsc STATUS, C
					goto $ -2
				if (dcnt % 4) >= 2
					if (dcnt % 4) == 3
						goto $ + 1
						nop
					else    ;(dcnt % 4) == 2
						goto $ + 1
					endif
				else
					if (dcnt % 4) == 1
						nop
					endif
				endif
			endif
		endif
	else
		if dcnt > D'14'
			ifdef _ContDelay	;Use Delay
				movlw dcnt - 1
				call Delay
			else				;Make loop
				movlw dcnt - 4
				addlw -4
				btfsc STATUS, C
					goto $ -2
				if (dcnt % 4) >= 2
					if (dcnt % 4) == 3
						goto $ + 1
						nop
					else    ;(dcnt % 4) == 2
						goto $ + 1
					endif
				else
					if (dcnt % 4) == 1
						nop
					endif
				endif
			endif
		else
			ifdef _ProcDelay
				if dcnt > 7
					if dcnt > D'10'			;14 to 11
						if dcnt > D'12'		;14 or 13
							if dcnt == D'14'
								call Delay14
							else
								call Delay13
							endif
						else				;12 or 11
							if dcnt == D'12'
								call Delay12
							else
								call Delay11
							endif
						endif
					else					;10 to 8
						if dcnt > 8			;10 or 9
							if dcnt == D'10'
								call Delay10
							else
								call Delay9
							endif
						else
							call Delay8
						endif
					endif
				else						;7 to 1
					if dcnt > 3				;7 to 4
						if dcnt > 5			;7,6
							if dcnt == 7
								call Delay7
							else
								call Delay6
							endif
						else				;5,4
							if dcnt == 5
								call Delay5
							else
								call Delay4
							endif
						endif
					else					;3 to 1
						while dcnt > 1	;Set of goto's and nop
							goto $ + 1
							dcnt = dcnt - 2
						endw
						if dcnt == 1
							nop
						endif
					endif
				endif
			else							;No ProcDelay
				if dcnt >= D'11'	;Make loop
					movlw dcnt - 4
					addlw -4
					btfsc STATUS, C
						goto $ -2
					if (dcnt % 4) >= 2
						if (dcnt % 4) == 3
							goto $ + 1
							nop
						else    ;(dcnt % 4) == 2
							goto $ + 1
						endif
					else
						if (dcnt % 4) == 1
							nop
						endif
					endif
				else
					while dcnt > 1	;Set of goto's and nop
						goto $ + 1
						dcnt = dcnt - 2
					endw
					if dcnt == 1
						nop
					endif
				endif
			endif
		endif
	endif
endm
